@@ -15,7 +15,7 @@ _LOGGER = logging.getLogger()
 
 
 class HonConnection:
-    def __init__(self, email, password, session=None) -> None:
+    def __init__(self, email="", password="", session=None) -> None:
         super().__init__()
         self._email = email
         self._password = password
@@ -26,7 +26,8 @@ class HonConnection:
 
     async def __aenter__(self):
         self._session = aiohttp.ClientSession()
-        await self.setup()
+        if self._email and self._password:
+            await self.setup()
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
@@ -94,6 +95,48 @@ class HonConnection:
                 return {}
             return result["payload"]["history"]
 
+    async def last_activity(self, device: HonDevice):
+        url = f"{const.API_URL}/commands/v1/retrieve-last-activity"
+        params = {"macAddress": device.mac_address}
+        async with self._session.get(url, params=params, headers=await self._headers) as response:
+            result = await response.json()
+            if result and (activity := result.get("attributes")):
+                return activity
+        return {}
+
+    async def appliance_configuration(self):
+        url = f"{const.API_URL}/config/v1/appliance-configuration"
+        headers = {"x-api-key": const.API_KEY, "content-type": "application/json"}
+        async with self._session.get(url, headers=headers) as response:
+            result = await response.json()
+            if result and (data := result.get("payload")):
+                return data
+        return {}
+
+    async def app_config(self, language="en", beta=True):
+        headers = {"x-api-key": const.API_KEY, "content-type": "application/json"}
+        url = f"{const.API_URL}/app-config"
+        payload = {
+              "languageCode": language,
+              "beta": beta,
+              "appVersion": const.APP_VERSION,
+              "os": const.OS
+            }
+        payload = json.dumps(payload, separators=(',', ':'))
+        async with self._session.post(url, headers=headers, data=payload) as response:
+            if (result := await response.json()) and (data := result.get("payload")):
+                return data
+        return {}
+
+    async def translation_keys(self, language="en"):
+        headers = {"x-api-key": const.API_KEY, "content-type": "application/json"}
+        config = await self.app_config(language=language)
+        if url := config.get("language", {}).get("jsonPath"):
+            async with self._session.get(url, headers=headers) as response:
+                if result := await response.json():
+                    return result
+        return {}
+
     async def load_attributes(self, device: HonDevice, loop=False):
         params = {
             "macAddress": device.mac_address,
